@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <torch/arg.h>
@@ -130,7 +131,7 @@ class ModuleHolder : torch::detail::ModuleHolderIndicator {
   /// NOTE: std::forward is qualified to prevent VS2017 emitting
   ///       error C2872: 'std': ambiguous symbol
   template <typename Arg>
-  decltype(auto) operator[](Arg&& arg) {
+  auto operator[](Arg&& arg) {
     return (*impl_)[::std::forward<Arg>(arg)];
   }
 
@@ -198,3 +199,7 @@ serialize::InputArchive& operator>>(
 
 /// Like `TORCH_MODULE_IMPL`, but defaults the `ImplType` name to `<Name>Impl`.
 #define TORCH_MODULE(Name) TORCH_MODULE_IMPL(Name, Name##Impl)
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
