@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 #include <ATen/core/ivalue.h>
@@ -65,7 +66,7 @@ TORCH_PYTHON_API py::object toPyObject(IValue ivalue);
 // Hack to overload the behavior of toIValue to accept Python
 // numbers in places where a Tensor is expected
 // See also torch::should_allow_numbers_as_tensors
-class ToIValueAllowNumbersAsTensors {
+class TORCH_PYTHON_API ToIValueAllowNumbersAsTensors {
   bool old_;
 
  public:
@@ -79,6 +80,10 @@ class ToIValueAllowNumbersAsTensors {
 // type of its field 'torch::jit::PythonFunctionGuard::func_'
 struct VISIBILITY_HIDDEN PythonFunctionGuard {
   explicit PythonFunctionGuard(py::function func) : func_(std::move(func)) {}
+  PythonFunctionGuard(const PythonFunctionGuard&) = delete;
+  PythonFunctionGuard(PythonFunctionGuard&&) = delete;
+  PythonFunctionGuard& operator=(const PythonFunctionGuard&) = delete;
+  PythonFunctionGuard& operator=(PythonFunctionGuard&&) = delete;
 
   ~PythonFunctionGuard() {
     pybind11::gil_scoped_acquire ag;
@@ -112,6 +117,9 @@ struct VISIBILITY_HIDDEN PythonFutureWrapper
 
   explicit PythonFutureWrapper(const PythonFutureWrapper&) = delete;
   PythonFutureWrapper& operator=(const PythonFutureWrapper&) = delete;
+  PythonFutureWrapper(PythonFutureWrapper&&) = default;
+  PythonFutureWrapper& operator=(PythonFutureWrapper&&) = default;
+  ~PythonFutureWrapper() = default;
 
   bool done() {
     return fut->completed();
@@ -275,6 +283,9 @@ struct VISIBILITY_HIDDEN PythonAwaitWrapper
 
   explicit PythonAwaitWrapper(const PythonAwaitWrapper&) = delete;
   PythonAwaitWrapper& operator=(const PythonAwaitWrapper&) = delete;
+  PythonAwaitWrapper(PythonAwaitWrapper&&) = default;
+  PythonAwaitWrapper& operator=(PythonAwaitWrapper&&) = default;
+  ~PythonAwaitWrapper() = default;
 
   py::object wait() {
     py::gil_scoped_acquire acquire;
@@ -342,7 +353,8 @@ inline TypedIValue toDictKeyIValue(py::handle key) {
   } else if (py::isinstance<py::float_>(key)) {
     return TypedIValue(py::cast<double>(key), FloatType::get());
   } else {
-    AT_ERROR("Dictionary inputs may only have string, int, or float keys");
+    TORCH_CHECK(
+        false, "Dictionary inputs may only have string, int, or float keys");
   }
 }
 
@@ -389,6 +401,10 @@ inline InferredType tryToInferType(py::handle input) {
     return InferredType(FloatType::get());
   } else if (PyComplex_CheckExact(input.ptr())) {
     return InferredType(ComplexType::get());
+    // NOLINTNEXTLINE(bugprone-branch-clone)
+  } else if (py::isinstance<py::bytes>(input)) {
+    // NOTE: We may need a ByteType in the future
+    return InferredType(StringType::get());
   } else if (py::isinstance<py::str>(input)) {
     return InferredType(StringType::get());
   } else if (THPLayout_Check(input.ptr())) {
@@ -418,20 +434,22 @@ inline InferredType tryToInferType(py::handle input) {
   }
 
   py::bool_ isClass =
-      py::module::import("inspect").attr("isclass")(input.get_type());
+      py::module::import("inspect").attr("isclass")(py::type::handle_of(input));
   if (py::cast<bool>(isClass)) {
     // Assume that the class is compiled already or will compile. Invalidate
     // this later if needed.
     bool class_compiled = true;
 
     // Check if the type is already compiled.
-    py::object existing_ty = py::module::import("torch.jit._state")
-                                 .attr("_get_script_class")(input.get_type());
+    py::object existing_ty =
+        py::module::import("torch.jit._state")
+            .attr("_get_script_class")(py::type::handle_of(input));
 
     if (existing_ty.is_none()) {
       // If not, try to compile it.
-      py::bool_ can_compile = py::module::import("torch._jit_internal")
-                                  .attr("can_compile_class")(input.get_type());
+      py::bool_ can_compile =
+          py::module::import("torch._jit_internal")
+              .attr("can_compile_class")(py::type::handle_of(input));
 
       if (py::cast<bool>(can_compile)) {
         // Try to compile the class. This is wrapped in a try-catch because
@@ -441,7 +459,7 @@ inline InferredType tryToInferType(py::handle input) {
         try {
           py::module::import("torch.jit._script")
               .attr("_recursive_compile_class")(
-                  input.get_type(), SourceRange());
+                  py::type::handle_of(input), SourceRange());
         } catch (...) {
           // Invalidate the assumption that the class compiled so that we don't
           // look up and return its JIT type as the type for the input.
@@ -453,8 +471,9 @@ inline InferredType tryToInferType(py::handle input) {
     // If the class compiled successfully, look up the existing JIT type by
     // qualified name and return it.
     if (class_compiled) {
-      auto script_class = py::module::import("torch.jit._state")
-                              .attr("_get_script_class")(input.get_type());
+      auto script_class =
+          py::module::import("torch.jit._state")
+              .attr("_get_script_class")(py::type::handle_of(input));
 
       if (!script_class.is_none()) {
         auto class_type = py::cast<ClassTypePtr>(script_class);
@@ -627,18 +646,18 @@ inline InferredType tryToInferContainerType(
           "are supported ",
           "as inputs or outputs of traced functions",
           ", but instead got value of type ",
-          py::str(input.get_type().attr("__name__")),
+          py::str(py::type::handle_of(input).attr("__name__")),
           "."));
     } else {
       // TODO: this message is not correct anymore, since this InferredType is
-      // used from a bunch of circumstances unrelated to tracing. We can re-use
+      // used from a bunch of circumstances unrelated to tracing. We can reuse
       // this instead of the attribute_failure stuff in concreteType
       return InferredType(c10::str(
-          "Only tensors and (possibly nested) tuples of tensors, lists, or dicts",
+          "Only tensors and (possibly nested) tuples of tensors, lists, or dicts ",
           "are supported ",
           "as inputs or outputs of traced functions",
           ", but instead got value of type ",
-          py::str(input.get_type().attr("__name__")),
+          py::str(py::type::handle_of(input).attr("__name__")),
           "."));
     }
   }
@@ -676,7 +695,7 @@ inline IValue toTypeInferredIValue(py::handle input) {
     if (auto mod = as_module(object)) {
       // if obj is already a ScriptModule, just return its ivalue
       auto ptr = mod.value()._ivalue();
-      // explict copy semantics for strong ownership of the resource.
+      // explicit copy semantics for strong ownership of the resource.
       return c10::intrusive_ptr<c10::ivalue::Object>::reclaim_copy(
           ptr.release());
     }
@@ -687,8 +706,12 @@ inline IValue toTypeInferredIValue(py::handle input) {
       return c10::intrusive_ptr<c10::ivalue::Object>::reclaim_copy(
           ptr.release());
     }
-    AT_ERROR(
-        "Tracer cannot infer type of ", py::str(input), "\n:", match.reason());
+    TORCH_CHECK(
+        false,
+        "Tracer cannot infer type of ",
+        py::str(input),
+        "\n:",
+        match.reason());
   }
   return toIValue(input, match.type());
 }
@@ -761,7 +784,7 @@ inline std::string friendlyTypeName(py::handle obj) {
     auto field_names =
         py::cast<std::vector<std::string>>(py::getattr(obj, "_fields"));
     std::stringstream ss;
-    ss << py::str(obj.get_type().attr("__name__"));
+    ss << py::str(py::type::handle_of(obj).attr("__name__"));
     ss << " (aka NamedTuple(";
     bool first = true;
     for (auto& field_name : field_names) {
@@ -774,7 +797,7 @@ inline std::string friendlyTypeName(py::handle obj) {
     ss << "))";
     return ss.str();
   } else {
-    return py::str(obj.get_type().attr("__name__"));
+    return py::str(py::type::handle_of(obj).attr("__name__"));
   }
 }
 
@@ -822,7 +845,7 @@ inline IValue returnToIValue(const TypePtr& type, py::handle object) {
         " expected value of type ",
         type->str(),
         " for return value but instead got value of type ",
-        py::str(object.get_type().attr("__name__")),
+        py::str(py::type::handle_of(object).attr("__name__")),
         ".",
         "\nValue: ",
         py::repr(object),
@@ -847,9 +870,9 @@ inline py::object getScriptedClassOrError(const c10::NamedTypePtr& classType) {
 
 struct VISIBILITY_HIDDEN tuple_slice {
   /*implicit*/ tuple_slice(py::tuple tup_)
-      : tup(std::move(tup_)), b(0), e(tup.size()) {}
+      : tup(std::move(tup_)), b(0), e(static_cast<int64_t>(tup.size())) {}
   tuple_slice(py::tuple tup_, int64_t b_)
-      : tup(std::move(tup_)), b(b_), e(tup.size()) {}
+      : tup(std::move(tup_)), b(b_), e(static_cast<int64_t>(tup.size())) {}
   tuple_slice(py::tuple tup_, int64_t b_, int64_t e_)
       : tup(std::move(tup_)), b(b_), e(e_) {}
   py::detail::tuple_iterator begin() const {
@@ -1056,6 +1079,7 @@ inline Stack createStackForSchema(
   return stack;
 }
 
+// NOLINTNEXTLINE(cppcoreguidelines-rvalue-reference-param-not-moved)
 inline py::object createPyObjectForStack(Stack&& stack) {
   if (stack.empty()) {
     return py::none();
@@ -1073,7 +1097,11 @@ inline py::object createPyObjectForStack(Stack&& stack) {
     return_values[ret] = toPyObject(std::move(stack[ret]));
   }
 
+#if defined(__clang__)
   return std::move(return_values);
+#else
+  return return_values;
+#endif
 }
 
 // TODO: Remove once we clean up the GraphExecutor usage.
@@ -1082,9 +1110,10 @@ inline Stack evilDeprecatedBadCreateStackDoNotUse(
     at::ArrayRef<Value*> inputs,
     size_t reserve_extra_space = 0) {
   if (tuple.size() != inputs.size()) {
-    AT_ERROR(
+    TORCH_CHECK(
+        false,
         "expected " + std::to_string(inputs.size()) + " inputs, but got " +
-        std::to_string(tuple.size()));
+            std::to_string(tuple.size()));
   }
   Stack result;
   result.reserve(tuple.size() + reserve_extra_space);
@@ -1249,12 +1278,27 @@ TORCH_PYTHON_API std::pair<std::shared_ptr<Operator>, Stack> getOpWithStack(
     const py::args& args,
     const py::kwargs& kwargs);
 
+// Efficient overload (does not require vector allocation) of the
+// above for use from C++ code.
+std::pair<std::shared_ptr<Operator>, Stack> getOpWithStack(
+    c10::ArrayRef<std::shared_ptr<Operator>> operations,
+    const py::args& args,
+    const py::kwargs& kwargs);
+
 TORCH_PYTHON_API py::object invokeOperatorFromPython(
     const std::vector<std::shared_ptr<Operator>>& operations,
     const py::args& args,
     const py::kwargs& kwargs,
     std::optional<c10::DispatchKey> dk = std::nullopt);
 
+// Efficient overload (does not require vector allocation) of the
+// above for use from C++ code.
+py::object invokeOperatorFromPython(
+    c10::ArrayRef<std::shared_ptr<Operator>> operations,
+    const py::args& args,
+    const py::kwargs& kwargs,
+    std::optional<c10::DispatchKey> dk = std::nullopt);
+
 TORCH_PYTHON_API std::optional<py::object> _maybe_handle_torch_function(
     const std::string& ns,
     const std::string& method_name,
@@ -1276,4 +1320,18 @@ TORCH_PYTHON_API py::object _get_operation_for_overload_or_packet(
     bool is_overload,
     std::optional<c10::DispatchKey> dk = std::nullopt);
 
+// Efficient overload (does not require vector allocation) of the
+// above for use from C++ code.
+py::object _get_operation_for_overload_or_packet(
+    c10::ArrayRef<std::shared_ptr<Operator>> operations,
+    Symbol symbol,
+    const py::args& args,
+    const py::kwargs& kwargs,
+    bool is_overload,
+    std::optional<c10::DispatchKey> dk = std::nullopt);
+
 } // namespace torch::jit
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
