@@ -1,13 +1,14 @@
 # mypy: allow-untyped-defs
-import torch
-from torch.overrides import (
-    handle_torch_function,
-    has_torch_function_unary,
-)
-from torch._C import _rename_privateuse1_backend, _get_privateuse1_backend_name
-from typing import Optional, Union
 
-__all__ = ["rename_privateuse1_backend", "generate_methods_for_privateuse1_backend"]
+import torch
+from torch._C import _get_privateuse1_backend_name, _rename_privateuse1_backend
+from torch.overrides import handle_torch_function, has_torch_function_unary
+
+
+__all__ = [
+    "rename_privateuse1_backend",
+    "generate_methods_for_privateuse1_backend",
+]
 
 # TODO: Should use `torch._C._get_privateuse1_backend_name()` to get
 # renamed-backend name for `privateuse1`, but the func will cause an
@@ -15,6 +16,7 @@ __all__ = ["rename_privateuse1_backend", "generate_methods_for_privateuse1_backe
 # `_privateuse1_backend_name`.
 _privateuse1_backend_name = "privateuseone"
 
+
 def rename_privateuse1_backend(backend_name: str) -> None:
     r"""
     Rename the privateuse1 backend device to make it more convenient to use as a device name within PyTorch APIs.
@@ -78,16 +80,22 @@ def rename_privateuse1_backend(backend_name: str) -> None:
     global _privateuse1_backend_name
     _privateuse1_backend_name = backend_name
 
-def _check_register_once(module, attr):
+
+def _check_register_once(module, attr) -> None:
     if hasattr(module, attr):
-        raise RuntimeError(f"The custom device module of {module} has already been registered with {attr}")
+        raise RuntimeError(
+            f"The custom device module of {module} has already been registered with {attr}"
+        )
 
 
-def _normalization_device(custom_backend_name: str, device: Optional[Union[int, str, torch.device]] = None) -> int:
+def _normalization_device(
+    custom_backend_name: str, device: int | str | torch.device | None = None
+) -> int:
     def _get_current_device_index():
         _get_device_index = "current_device"
-        if hasattr(torch, custom_backend_name) and \
-                hasattr(getattr(torch, custom_backend_name), _get_device_index):
+        if hasattr(torch, custom_backend_name) and hasattr(
+            getattr(torch, custom_backend_name), _get_device_index
+        ):
             return getattr(getattr(torch, custom_backend_name), _get_device_index)()
         else:
             # The default device index is 0.
@@ -100,7 +108,7 @@ def _normalization_device(custom_backend_name: str, device: Optional[Union[int,
     elif isinstance(device, str):
         device = torch.device(device)
 
-    # variable devcie can only be torch.device type or int type
+    # variable device can only be torch.device type or int type
     if isinstance(device, torch.device):
         if device.type != custom_backend_name:
             raise RuntimeError(f"Invalid device, must be {custom_backend_name} device")
@@ -122,12 +130,16 @@ def _generate_tensor_methods_for_privateuse1_backend(custom_backend_name: str) -
             return handle_torch_function(wrap_tensor_backend.__get__, (self,), self)  # type: ignore[attr-defined]
         return self.device.type == custom_backend_name
 
-    _check_register_once(torch.Tensor, f'is_{custom_backend_name}')
-    wrap_tensor_backend.fget.__name__ = f'is_{custom_backend_name}'  # type: ignore[attr-defined]
-    setattr(torch.Tensor, f'is_{custom_backend_name}', wrap_tensor_backend)
+    _check_register_once(torch.Tensor, f"is_{custom_backend_name}")
+    wrap_tensor_backend.fget.__name__ = f"is_{custom_backend_name}"  # type: ignore[attr-defined]
+    setattr(torch.Tensor, f"is_{custom_backend_name}", wrap_tensor_backend)
 
-    def wrap_tensor_to(self: torch.Tensor, device: Optional[Union[int, torch.device]] = None, non_blocking=False,
-                       **kwargs) -> torch.Tensor:
+    def wrap_tensor_to(
+        self: torch.Tensor,
+        device: int | torch.device | None = None,
+        non_blocking=False,
+        **kwargs,
+    ) -> torch.Tensor:
         r"""Perform Tensor device conversion. Call the to operator implementation.
 
         .. note::
@@ -143,9 +155,20 @@ def _generate_tensor_methods_for_privateuse1_backend(custom_backend_name: str) -
             **kwargs (dict): For compatibility, may contain the key ``memory_format`` argument.
         """
         if has_torch_function_unary(self):
-            return handle_torch_function(wrap_tensor_to, (self,), self, device=device, non_blocking=False, **kwargs)
+            return handle_torch_function(
+                wrap_tensor_to,
+                (self,),
+                self,
+                device=device,
+                non_blocking=False,
+                **kwargs,
+            )
         device_idx = _normalization_device(custom_backend_name, device)
-        return self.to(device=torch.device(f'{custom_backend_name}:{device_idx}'), non_blocking=non_blocking, **kwargs)
+        return self.to(
+            device=torch.device(f"{custom_backend_name}:{device_idx}"),
+            non_blocking=non_blocking,
+            **kwargs,
+        )
 
     _check_register_once(torch.Tensor, custom_backend_name)
     wrap_tensor_to.__name__ = custom_backend_name
@@ -159,10 +182,13 @@ def _generate_module_methods_for_privateuse1_backend(custom_backend_name: str) -
         raise RuntimeError(
             f"Can not automatically generate {custom_backend_name}() method for torch.nn.Module."
             f"Because torch.Tensor doesn't has the method {custom_backend_name}()."
-            f"For this error, you can try setting for_tensor=True.")
+            f"For this error, you can try setting for_tensor=True."
+        )
 
-    def wrap_module_to(self: torch.nn.modules.module.T,
-                       device: Optional[Union[int, torch.device]] = None) -> torch.nn.modules.module.T:
+    def wrap_module_to(
+        self: torch.nn.modules.module.T,
+        device: int | torch.device | None = None,
+    ) -> torch.nn.modules.module.T:
         r"""Move all model parameters and buffers to the custom device.
 
         This also makes associated parameters and buffers different objects. So
@@ -175,32 +201,43 @@ def _generate_module_methods_for_privateuse1_backend(custom_backend_name: str) -
         Args:
             device (int, optional): if specified, all parameters will be copied to that device
         """
+        # pyrefly: ignore [missing-attribute]
         return self._apply(lambda t: getattr(t, custom_backend_name)(device))
 
     _check_register_once(torch.nn.Module, custom_backend_name)
     setattr(torch.nn.Module, custom_backend_name, wrap_module_to)
 
-def _generate_packed_sequence_methods_for_privateuse1_backend(custom_backend_name: str) -> None:
+
+def _generate_packed_sequence_methods_for_privateuse1_backend(
+    custom_backend_name: str,
+) -> None:
     # Generate PackedSequence Module attributes and methods depends on Tensor methods,
     # so we need to check whether Tensor methods is already registered.
-    if not hasattr(torch.Tensor, f'is_{custom_backend_name}') or \
-       not hasattr(torch.Tensor, custom_backend_name):
+    if not hasattr(torch.Tensor, f"is_{custom_backend_name}") or not hasattr(
+        torch.Tensor, custom_backend_name
+    ):
         raise RuntimeError(
             f"Can not automatically generate is_{custom_backend_name}() or "
             f"{custom_backend_name}() method for torch.nn.utils.rnn.PackedSequence."
             f"Because torch.Tensor doesn't has the method is_{custom_backend_name}()"
             f"or {custom_backend_name}()."
-            f"For this error, you can try setting for_tensor=True.")
+            f"For this error, you can try setting for_tensor=True."
+        )
 
     @property  # type: ignore[misc]
     def wrap_tensor_backend(self: torch.nn.utils.rnn.PackedSequence) -> bool:
         return self.data.device.type == custom_backend_name
 
-    _check_register_once(torch.nn.utils.rnn.PackedSequence, f'is_{custom_backend_name}')
-    setattr(torch.nn.utils.rnn.PackedSequence, f'is_{custom_backend_name}', wrap_tensor_backend)
+    _check_register_once(torch.nn.utils.rnn.PackedSequence, f"is_{custom_backend_name}")
+    setattr(
+        torch.nn.utils.rnn.PackedSequence,
+        f"is_{custom_backend_name}",
+        wrap_tensor_backend,
+    )
 
-    def wrap_module_to(self: torch.nn.utils.rnn.PackedSequence,
-                       *args, **kwargs) -> torch.nn.utils.rnn.PackedSequence:
+    def wrap_module_to(
+        self: torch.nn.utils.rnn.PackedSequence, *args, **kwargs
+    ) -> torch.nn.utils.rnn.PackedSequence:
         r"""Move all model parameters and buffers to the custom device.
 
         This also makes associated parameters and buffers different objects. So
@@ -213,17 +250,25 @@ def _generate_packed_sequence_methods_for_privateuse1_backend(custom_backend_nam
         Args:
             device (int, optional): if specified, all parameters will be copied to that device
         """
-        ex = torch.tensor((), dtype=self.data.dtype, device=self.data.device).to(*args, **kwargs)
+        ex = torch.tensor((), dtype=self.data.dtype, device=self.data.device).to(
+            # pyrefly: ignore [not-iterable]
+            *args,
+            **kwargs,
+        )
         if ex.device.type == custom_backend_name:
+            # pyrefly: ignore [not-iterable]
             return self.to(*args, **kwargs)
-        kwargs.update({'device': custom_backend_name})
+        kwargs.update({"device": custom_backend_name})
+        # pyrefly: ignore [not-iterable]
         return self.to(*args, **kwargs)
 
     _check_register_once(torch.nn.utils.rnn.PackedSequence, custom_backend_name)
     setattr(torch.nn.utils.rnn.PackedSequence, custom_backend_name, wrap_module_to)
 
-def _generate_storage_methods_for_privateuse1_backend(custom_backend_name: str,
-                                                      unsupported_dtype: Optional[list[torch.dtype]] = None) -> None:
+
+def _generate_storage_methods_for_privateuse1_backend(
+    custom_backend_name: str, unsupported_dtype: list[torch.dtype] | None = None
+) -> None:
     # Attribute is registered in the _StorageBase class
     # and UntypedStorage obtains through inheritance.
     @property  # type: ignore[misc]
@@ -231,8 +276,10 @@ def _generate_storage_methods_for_privateuse1_backend(custom_backend_name: str,
         r"""Return the internal :class:`torch.UntypedStorage`."""
         return self.device.type == custom_backend_name
 
-    _check_register_once(torch.storage._StorageBase, f'is_{custom_backend_name}')
-    setattr(torch.storage._StorageBase, f'is_{custom_backend_name}', wrap_storage_backend)
+    _check_register_once(torch.storage._StorageBase, f"is_{custom_backend_name}")
+    setattr(
+        torch.storage._StorageBase, f"is_{custom_backend_name}", wrap_storage_backend
+    )
 
     def wrap_storage_to(self, device=None, non_blocking=False):
         r"""Return a copy of this object in custom device memory.
@@ -250,16 +297,18 @@ def _generate_storage_methods_for_privateuse1_backend(custom_backend_name: str,
         # but it depends on the extended function, so this part is temporarily omitted in the automatic generation.
         device_idx = _normalization_device(custom_backend_name, device)
 
-        if getattr(self, f'is_{custom_backend_name}'):
+        if getattr(self, f"is_{custom_backend_name}"):
             # storage has already on expected device.
             if self.get_device() == device_idx:
                 return self
         # For sparse storage, custom need to extend the implementation by themselves.
         if self.is_sparse:
-            raise RuntimeError(f"Can not support a sparse storage move to {custom_backend_name} backend")
+            raise RuntimeError(
+                f"Can not support a sparse storage move to {custom_backend_name} backend"
+            )
         # create untyped_storage and copy data
         untyped_storage = torch.UntypedStorage(
-            self.size(), device=torch.device(f'{custom_backend_name}:{device_idx}')
+            self.size(), device=torch.device(f"{custom_backend_name}:{device_idx}")
         )
         untyped_storage.copy_(self, non_blocking)
         return untyped_storage
@@ -275,27 +324,38 @@ def _generate_storage_methods_for_privateuse1_backend(custom_backend_name: str,
         torch.storage._warn_typed_storage_removal()
         return self._untyped_storage.device.type == custom_backend_name
 
-    _check_register_once(torch.TypedStorage, f'is_{custom_backend_name}')
-    setattr(torch.storage.TypedStorage, f'is_{custom_backend_name}', wrap_typed_storage_backend)
+    _check_register_once(torch.TypedStorage, f"is_{custom_backend_name}")
+    setattr(
+        torch.storage.TypedStorage,
+        f"is_{custom_backend_name}",
+        wrap_typed_storage_backend,
+    )
 
-    def wrap_typed_storage_to(self: torch.storage.TypedStorage,
-                              device=None, non_blocking=False, **kwargs) -> torch.storage.TypedStorage:
+    def wrap_typed_storage_to(
+        self: torch.storage.TypedStorage, device=None, non_blocking=False, **kwargs
+    ) -> torch.storage.TypedStorage:
         torch.storage._warn_typed_storage_removal()
         if unsupported_dtype and self.dtype in unsupported_dtype:
-            raise RuntimeError(f"Cannot create {custom_backend_name} storage "
-                               f"as {self.dtype} dtype is not supported by this backend")
+            raise RuntimeError(
+                f"Cannot create {custom_backend_name} storage "
+                f"as {self.dtype} dtype is not supported by this backend"
+            )
         custom_backend_storage: torch.UntypedStorage = getattr(
-            self._untyped_storage, custom_backend_name)(device, non_blocking, **kwargs)
+            self._untyped_storage, custom_backend_name
+        )(device, non_blocking, **kwargs)
         return self._new_wrapped_storage(custom_backend_storage)
 
     _check_register_once(torch.TypedStorage, custom_backend_name)
     setattr(torch.TypedStorage, custom_backend_name, wrap_typed_storage_to)
 
 
-def generate_methods_for_privateuse1_backend(for_tensor: bool = True, for_module: bool = True,
-                                             for_packed_sequence: bool = True,
-                                             for_storage: bool = False,
-                                             unsupported_dtype: Optional[list[torch.dtype]] = None) -> None:
+def generate_methods_for_privateuse1_backend(
+    for_tensor: bool = True,
+    for_module: bool = True,
+    for_packed_sequence: bool = True,
+    for_storage: bool = False,
+    unsupported_dtype: list[torch.dtype] | None = None,
+) -> None:
     r"""
     Automatically generate attributes and methods for the custom backend after rename privateuse1 backend.
 
@@ -337,11 +397,14 @@ def generate_methods_for_privateuse1_backend(for_tensor: bool = True, for_module
         _generate_module_methods_for_privateuse1_backend(custom_backend_name)
 
     if for_storage:
-        _generate_storage_methods_for_privateuse1_backend(custom_backend_name, unsupported_dtype)
+        _generate_storage_methods_for_privateuse1_backend(
+            custom_backend_name, unsupported_dtype
+        )
 
     if for_packed_sequence:
         _generate_packed_sequence_methods_for_privateuse1_backend(custom_backend_name)
 
+
 def _get_custom_mod_func(func_name: str):
     r"""
     Return the func named `func_name` defined in custom device module. If not defined,
@@ -370,13 +433,89 @@ def _get_custom_mod_func(func_name: str):
     it is marked as private. It is a convenience function for backend implementers to
     more easily call the hooks into their backend extensions.
     """
-    assert isinstance(func_name, str), f"func_name must be `str`, but got `{type(func_name)}`."
+    if not isinstance(func_name, str):
+        raise AssertionError(f"func_name must be `str`, but got `{type(func_name)}`.")
     backend_name = _get_privateuse1_backend_name()
-    custom_device_mod = getattr(torch, backend_name, None)  # type: ignore[arg-type]
-    function = getattr(custom_device_mod, func_name, None)  # type: ignore[arg-type]
+    custom_device_mod = getattr(torch, backend_name, None)
+    function = getattr(custom_device_mod, func_name, None)
     if custom_device_mod is None or function is None:
-        message = f'Try to call torch.{backend_name}.{func_name}. The backend must register a custom backend '
+        message = f"Try to call torch.{backend_name}.{func_name}. The backend must register a custom backend "
         message += f"module with `torch._register_device_module('{backend_name}', BackendModule)`. And "
         message += f"BackendModule needs to have the following API's:\n `{func_name}(*args, **kwargs)`. \n"
         raise RuntimeError(message)
     return function
+
+
+class _DummyBackendModule:
+    def is_initialized(self) -> bool:
+        return True
+
+    def is_available(self) -> bool:
+        return True
+
+    def current_device(self) -> int:
+        return 0
+
+    def _is_in_bad_fork(self) -> bool:
+        return False
+
+    def manual_seed_all(self, seed: int) -> None:
+        pass
+
+    def device_count(self) -> int:
+        return 1
+
+
+class _DummyPrivateUse1Hook(torch._C._acc.PrivateUse1Hooks):
+    def is_available(self) -> bool:
+        return True
+
+    def has_primary_context(self, dev_id) -> bool:
+        return True
+
+    def is_built(self) -> bool:
+        return True
+
+
+class _DummyDeviceGuard(torch._C._acc.DeviceGuard):
+    def type_(self):
+        return torch._C._autograd.DeviceType.PrivateUse1
+
+
+def _setup_privateuseone_for_python_backend(
+    rename=None, backend_module=None, hook=None, device_guard=None
+) -> None:
+    """This function will prepare the PrivateUse1 dispatch key to be used as a python backend.
+
+    WARNING: this API is experimental and might change without notice.
+
+    Formally, this registers things that Pytorch expects a registered backend
+    in C++ to have: including device guards, hooks, and backend modules and what not.
+
+    after this call, one can use `torch.library` to write Ops for this dispatch key
+    and expect it to behave like a backend registered in C++.
+
+    See the unit test at test/test_privateuseone_python_backend.py for more details.
+
+    Args:
+        rename: str | None, if passed in, we will rename privateuseone backend to
+           the name given.
+        backend_module: object | None, if passed in None, we will use DummyBackendModule
+        hook: object | None, if passed in None, we will use DummyPrivateUse1Hook
+        device_guard: object | None, if passed in None, we will use DummyDeviceGuard
+    """
+    # NOTE: the ordering of which these functions are called is important.
+    if rename is not None:
+        torch.utils.rename_privateuse1_backend(rename)
+    else:
+        rename = "privateuseone"
+    torch.utils.generate_methods_for_privateuse1_backend()
+    if backend_module is None:
+        backend_module = _DummyBackendModule()
+    if hook is None:
+        hook = _DummyPrivateUse1Hook()
+    if device_guard is None:
+        device_guard = _DummyDeviceGuard()
+    torch._register_device_module(rename, backend_module)
+    torch._C._acc.register_python_privateuseone_hook(hook)
+    torch._C._acc.register_python_privateuseone_device_guard(device_guard)
